//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "NewNymDlgs.h"
#include "PGPsdk.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNewNymDlg *NewNymDlg;
//---------------------------------------------------------------------------

bool __fastcall GetNewNymValues(TForm *Owner, TPGPsdk *PGP, AnsiString &PGPKey, bool &ReplyBlock,
   AnsiString &Address, int &ChainLength)
{
   if (SecureModeMsg(Owner))
      return false;

   NewNymDlg = new TNewNymDlg(Owner, PGPKey, ReplyBlock, Address, ChainLength);
   AnsiString defs = Global->DB->GetIni("nymfullautodefs", "1,,3");

   NewNymDlg->ReplyBlock->Checked = GetTok(0, defs, ',').ToIntDef(0);
   NewNymDlg->OrgRB = NewNymDlg->ReplyBlock->Checked;
   NewNymDlg->Address->Text = GetTok(1, defs, ',');
   NewNymDlg->OrgAddress = NewNymDlg->Address->Text;
   NewNymDlg->ChainLength->ItemIndex = GetTok(2, defs, ',').ToIntDef(2);
   NewNymDlg->OrgLength = NewNymDlg->ChainLength->ItemIndex;

   TStringList *keys = new TStringList();
   keys->Sorted = true;

   PGP->GetKeysLV(PGPPublicRing(), PGPSecretRing(), keys);

   for (int i = 0; i < keys->Count; i++)
   {
      AnsiString key = keys->Strings[i];

      if (GetTok(6, key, '\255') == "0")  // secret ?
         continue;

      if (GetTok(4, key, '\255').LowerCase().Pos("rsa legacy") == 1 &&  // algo
          GetTok(3, key, '\255').ToIntDef(2049) > 2048)                 // bits
         continue;

      NewNymDlg->PGPKey->Items->Add(GetTok(1, key, '\255') + " " + GetTok(0, key, '\255'));
   }

   NewNymDlg->PGPKey->Items->Insert(0, "select here ---------------------------------------------");
   NewNymDlg->PGPKey->ItemIndex = 0;

   delete keys;

   NewNymDlg->ActiveControl = NewNymDlg->PGPKey;
   NewNymDlg->ShowModal();
   bool ok = NewNymDlg->ModalResult == mrOk;
   {
      PGPKey = NewNymDlg->PGPKey->Items->Strings[NewNymDlg->PGPKey->ItemIndex];
      ReplyBlock = NewNymDlg->ReplyBlock->Checked;
      Address = NewNymDlg->Address->Text;
      ChainLength = NewNymDlg->ChainLength->ItemIndex + 1;
      defs = IntToStr(ReplyBlock) + "," + Address + "," + IntToStr(ChainLength - 1);
      Global->DB->PutIni("nymfullautodefs", defs);
   }
   delete NewNymDlg;

   return ok;
}

//---------------------------------------------------------------------------
// NewNymDlg
//---------------------------------------------------------------------------

__fastcall TNewNymDlg::TNewNymDlg(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------

__fastcall TNewNymDlg::TNewNymDlg(TComponent* Owner, AnsiString &PgpKey, bool &replyblock,
   AnsiString &address, int &chainlength) : TForm(Owner)
{
   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, PGPKey->Font, NULL);
   PGPKey->Color = color;

   ReplyBlock->Width = Canvas->TextWidth(ReplyBlock->Caption + 20);
   if (chainlength > 0)
      ChainLength->Text = IntToStr(chainlength);
   ReplyBlock->Checked = replyblock;
   Address->Text = address;

   PGPKey->ItemHeight = abs(PGPKey->Font->Height) + 5;

   nymservers = new TStringList();
   nymservers->Duplicates = dupIgnore;
   GetNymServers(nymservers);
}
//---------------------------------------------------------------------------

__fastcall TNewNymDlg::~TNewNymDlg()
{
   delete nymservers;
}
//---------------------------------------------------------------------------

bool __fastcall TNewNymDlg::Modified()
{
   bool mod = false;

   if (!mod) mod = ChainLength->ItemIndex != OrgLength;
   if (!mod) mod = ReplyBlock->Checked != OrgRB;
   if (!mod) mod = Address->Text != OrgAddress;
   if (!mod) mod = PGPKey->ItemIndex != 0;

   return mod;
}
//---------------------------------------------------------------------------

void __fastcall TNewNymDlg::ReplyBlockClick(TObject *Sender)
{
   bool enable = ReplyBlock->Checked;
   TColor parentcolor = ((TForm*)Owner)->Color;

   Address->Color = enable ? clWindow : parentcolor;
   ChainLength->Color = enable ? clWindow : parentcolor;

   Address->Enabled = enable;
   ChainLength->Enabled = enable;
   Label3->Enabled = enable;
   Label2->Enabled = enable;

   if (enable)
      ActiveControl = Address;
}
//---------------------------------------------------------------------------

bool __fastcall TNewNymDlg::CheckIsNymInUse()
{
   bool is = false;
   AnsiString nym = GetEAddress(PGPKey->Text);
   TStringList *nyms = new TStringList();
   Global->DB->GetNyms(nyms);

   for (int i = 0; i < nyms->Count; i++)
   {
      if (nyms->Strings[i].Pos(nym) > 0)
      {
         is = true;
         break;
      }
   }
   delete nyms;

   if (is && ReplyBlock->Checked)
   {
      if (MsgBox(this,
         "This nym is already in use! You can use this key with another nym,    \n"
         "but I cannot create a reply block for you, at this point. Do you want    \n"
         "to continue with this key?", MT_INFO, MB_OKCANCEL) == IDOK)
      {
         ReplyBlock->Checked = false;
         Snooze(300);
         is = false;
      }
   }

   return is;
}
//---------------------------------------------------------------------------

bool __fastcall TNewNymDlg::CheckNotNymDomain()
{
   AnsiString nym = GetEAddress(PGPKey->Text);
   AnsiString nymdomain = nym.SubString(nym.Pos("@") + 1, 256);
   bool isdom = nymservers->IndexOf(nymdomain) >= 0;
   if (!isdom && ReplyBlock->Checked)
   {
      if (MsgBox(this,
         "The domain in your nym's email address is not an existing nym domain.    \n"
         "You can use this key but I cannot create a reply block for you, at this    \n"
         "point. Do you want to continue with this key?",
          MT_INFO, MB_OKCANCEL) == IDOK)
      {
         ReplyBlock->Checked = false;
         Snooze(300);
         isdom = true;
      }
   }

   return !isdom;
}
//---------------------------------------------------------------------------

void __fastcall TNewNymDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult == mrOk)
   {
      if (SecureModeMsg(this))
      {
         CanClose = false;
         return;
      }

      if (PGPKey->ItemIndex == 0 || PGPKey->Text.IsEmpty())
      {
         MsgBox(this,
            "You need to select a PGP key or click Cancel.    ",
             MT_INFO, MB_OK);
         CanClose = false;
         return;
      }

      if (ReplyBlock->Checked)
      {
         if (Address->Text.IsEmpty() || ChainLength->Text.IsEmpty())
         {
            MsgBox(this,
              "You have 'Reply Block' ticked. You need to provide an email address    \n"
              "and chain length.", MT_INFO, MB_OK);
            CanClose = false;
            return;
         }

         int p = Address->Text.Pos("@");
         if (p < 2 || p > Address->Text.Length() - 3)
         {
            MsgBox(this,
               "You have given me an email address but it is not a valid address.    \n"
               "Please take another look. I see it does not have an '@' symbol.",
                MT_ERROR, MB_OK);
            CanClose = false;
            return;
         }

         if (CheckIsNymInUse())
         {
            CanClose = false;
            return;
         }

         if (CheckNotNymDomain())
         {
            CanClose = false;
            return;
         }
      }

      /*AnsiString address, server;
      GetAddressAndServer(PGPKey->Text, address, server);
      if (IsNymserver(server) && !ValidKeyForNymServer(PGPKey->Text, server))
      {
         MsgBox(this,
            "The key you have selectd will not work with nymserver in the    \n"
            "key's email address. That nymserver supports only RSA keys with    \n"
            "2048 or less bits.", MT_INFO, MB_OK)
         CanClose = false;
         return;
      }*/
   }

   if (ModalResult == mrCancel)
   {
      if (Modified())
      {
         if (MsgBox(this,
           "You have modified your info. Are you sure you want to cancel?    ",
            MT_CONFIRM, MB_YESNO) == IDNO)
         {
            CanClose = false;
            return;
         }
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TNewNymDlg::PGPKeyDrawItem(TWinControl *Control, int Index,
   TRect &Rect, TOwnerDrawState State)
{
   TCanvas *canvas = PGPKey->Canvas;
   AnsiString keyid, name, key = PGPKey->Items->Strings[Index];
   if (Index == 0)
   {
      canvas->TextRect(Rect, Rect.Left + 2, Rect.Top + 1, key);
      return;
   }

   keyid = key.SubString(1, 10);
   name = key.SubString(12, 256);

   AnsiString orgname = canvas->Font->Name;
   canvas->Font->Name = "Courier New";
   int idwidth = canvas->TextWidth(keyid);

   canvas->TextRect(Rect, Rect.Left + 2, Rect.Top + 1, keyid);
   canvas->Font->Name = orgname;
   Rect.Left += idwidth + 12;
   canvas->TextRect(Rect, Rect.Left, Rect.Top + 1, name);
}
//---------------------------------------------------------------------------

void __fastcall TNewNymDlg::GetNymServers(TStrings *servers)
{
   assert(servers != NULL);

   servers->Clear();
   AnsiString serverfile = AppDir() + NymserverFile;
   if (FileExists(serverfile))
   {
      TStringList *lines = new TStringList();
      lines->LoadFromFile(serverfile);
      for (int i = 0; i < lines->Count; i++)
      {
         AnsiString line = lines->Strings[i];

         if (line.Pos("$nymserver{") == 1)
         {
            AnsiString nymserver = GetTok(0, GetTok(1, line, '@'), '>');
            if (servers->IndexOf(nymserver) < 0)
               servers->Add(nymserver);
            continue;
         }

         //if (line.Pos("-----BEGIN PGP") == 1)
         //   break;
      }

      delete lines;
   }
}
//---------------------------------------------------------------------------


