//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "NymPrintDlgs.h"
#include "NymProfiles.h"
#include "printdlgs2.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TNymPrintDlg *NymPrintDlg;
//---------------------------------------------------------------------------

void __fastcall PrintNyms(TForm *Owner, TNymsFrame2 *nymframe)
{
   if (SecureModeMsg(Owner))
      return;

   NymPrintDlg = new TNymPrintDlg(Owner);
   NymPrintDlg->Font->Assign(Owner->Font);
 //TNymsFrame2 *nymframe = Owner->DetailFrame;

   int hassels = nymframe->ListView->SelCount > 0;
   NymPrintDlg->SelectedBox->Checked = hassels;
   NymPrintDlg->SelectedBox->Enabled = hassels;
   NymPrintDlg->ShowModal();

   if (NymPrintDlg->ModalResult == mrOk)
   {
      TNymProfile *nym = new TNymProfile();
      TStringList *data = new TStringList();
      TStringList *nyms = new TStringList();

      for (int i = 0; i < nymframe->ListView->Items->Count; i++)
      {
         TListItem *item = nymframe->ListView->Items->Item[i];
         if (!hassels || (NymPrintDlg->SelectedBox->Checked && item->Selected))
         {
            Global->DB->GetNym(nymframe->GetNymIndexStr(item), nym);
            assert(nym->Count > 0);

            nyms->Add(item->Caption);
            nyms->Add("=============================================================================");
            if (NymPrintDlg->HistoryBox->Checked)
            {
               if (NymPrintDlg->CurrentBox->Checked) nym->GetHistory(0, data);
               else nym->GetHistory(data);

               bool prunepub = !NymPrintDlg->PGPBox->Checked || !NymPrintDlg->PublicBox->Checked;
               bool prunesec = !NymPrintDlg->PGPBox->Checked || !NymPrintDlg->SecretBox->Checked;
               if (prunepub)
                  NymPrintDlg->PruneKey("Public", data);
               if (prunesec)
                  NymPrintDlg->PruneKey("Secret", data);

               assert(data->Count > 0);
               nyms->AddStrings(data);

               if (NymPrintDlg->PGPBox->Checked)
               {
                  if ((prunepub && !prunesec) || (!prunepub && !prunesec))
                  {
                     nyms->Add("");
                  }
               }
            }

          //if (NymPrintDlg->ReplayBox->Checked)
          //{
          //   nym->GetReplay(data);
          //   nyms->AddStrings(data);
          //   nyms->Add("");
          //}

            if (NymPrintDlg->LogBox->Checked)
            {
               nym->GetLog(data);
               assert(data->Count > 0);
               WrapText(data, 76);
               nyms->AddStrings(data);
               nyms->Add("");
            }

            if (NymPrintDlg->NotesBox->Checked)
            {
               nym->GetNotes(data);
               WrapText(data, 76);
               nyms->AddStrings(data);
               nyms->Add("");
            }

            nyms->Add("");
            nyms->Add("");
         }
      }

      if (nyms->Count > 0)
      {
         TFont *font = new TFont();
         font->Name = "Courier New";
         font->Size = 10;
//look(nyms);
         //PrintDocument(Application->MainForm, "QuickSilver Nyms", nyms, font);

         delete font;
      }
      delete nyms;
      delete data;
      delete nym;
   }

   delete NymPrintDlg;
}
//---------------------------------------------------------------------------

void __fastcall TNymPrintDlg::PruneKey(const AnsiString &which, TStrings *history)
{
   int i = 0; while (i < history->Count - 1)
   {
      if (history->Strings[i].Pos(which + " Key: ") == 1)
      {
         history->Delete(i);

         while (i < history->Count)
         {
            if (history->Strings[i].Pos("-----END PGP "))
            {
               history->Delete(i);

               while (i < history->Count && history->Strings[i].Trim().IsEmpty())
                  history->Delete(i);

               break;
            }

            history->Delete(i);
         }
      }
      i++;
   }
}
//---------------------------------------------------------------------------

__fastcall TNymPrintDlg::TNymPrintDlg(TComponent* Owner)
        : TForm(Owner)
{
   TStringList *checked = new TStringList();
   checked->CommaText = Global->DB->GetIni("printnymsdef",
      "history=1,current=1,all=0,pgpkey=0,public=0,secret=0,replay=0,log=0,notes=0");

   PGPBox->OnClick = NULL;
   PublicBox->OnClick = NULL;
   SecretBox->OnClick = NULL;

   bool history = checked->Values["history"] == "1";
   HistoryBox->Checked = history;
   CurrentBox->Checked = checked->Values["current"] == "1";
   CurrentBox->Enabled = history;
   AllBox->Checked = checked->Values["all"] == "1";
   AllBox->Enabled = history;

   bool pgp = checked->Values["pgpkey"] == "1";
   PGPBox->Checked = pgp;
   PGPBox->Enabled = history;
   PublicBox->Checked = checked->Values["public"] == "1";
   PublicBox->Enabled = history && pgp;
   SecretBox->Checked = checked->Values["secret"] == "1";
   SecretBox->Enabled = history && pgp;

   SecretBox->OnClick = SecretBoxClick;
   PublicBox->OnClick = PublicBoxClick;
   PGPBox->OnClick = PGPBoxClick;

   ReplayBox->Checked = checked->Values["replay"] == "1";
   NotesBox->Checked = checked->Values["notes"] == "1";
   LogBox->Checked = checked->Values["log"] == "1";

   delete checked;

   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, NULL, NULL);
}
//---------------------------------------------------------------------------

__fastcall TNymPrintDlg::~TNymPrintDlg()
{
   TStringList *checked = new TStringList();

   checked->Values["history"] = HistoryBox->Checked ? "1" : "0";
   checked->Values["current"] = CurrentBox->Checked ? "1" : "0";
   checked->Values["pgpkey"]  = PGPBox->Checked ? "1" : "0";
   checked->Values["public"]  = PublicBox->Checked ? "1" : "0";
   checked->Values["secret"]  = SecretBox->Checked ? "1" : "0";
   checked->Values["all"]     = AllBox->Checked ? "1" : "0";
   checked->Values["replay"]  = ReplayBox->Checked ? "1" : "0";
   checked->Values["log"]     = LogBox->Checked ? "1" : "0";
   checked->Values["notes"]   = NotesBox->Checked ? "1" : "0";

   Global->DB->PutIni("printnymsdef", checked->CommaText);
   delete checked;
}
//---------------------------------------------------------------------------

void __fastcall TNymPrintDlg::FormCloseQuery(TObject *, bool &CanClose)
{
   if (ModalResult == mrOk)
   {
      if (!HistoryBox->Checked && !ReplayBox->Checked && !LogBox->Checked &&
          !NotesBox->Checked)
      {
         MsgBox(this, "No items selected for printing!", MT_INFO, MB_OK);
         CanClose = false;
         return;
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymPrintDlg::HistoryBoxClick(TObject *)
{
   bool history = HistoryBox->Checked;
   CurrentBox->Enabled = history;
   AllBox->Enabled = history;
   bool pgp = PGPBox->Checked;
   PGPBox->Enabled = history;
   PublicBox->Enabled = history && pgp;
   SecretBox->Enabled = history && pgp;
}
//---------------------------------------------------------------------------

void __fastcall TNymPrintDlg::PGPBoxClick(TObject *)
{
   bool checked = PGPBox->Checked;
   if (checked && !PublicBox->Checked && !SecretBox->Checked)
      PublicBox->Checked = true;
   PublicBox->Enabled = checked;
   SecretBox->Enabled = checked;
}
//---------------------------------------------------------------------------

void __fastcall TNymPrintDlg::PublicBoxClick(TObject *)
{
   if (!PublicBox->Checked && !SecretBox->Checked)
   {
      PublicBox->Enabled = false;
      SecretBox->Enabled = false;
      PGPBox->Checked = false;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymPrintDlg::SecretBoxClick(TObject *)
{
   if (!SecretBox->Checked && !PublicBox->Checked)
   {
      SecretBox->Enabled = false;
      PublicBox->Enabled = false;
      PGPBox->Checked = false;
   }
}
//---------------------------------------------------------------------------

