//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "Global2.h"
#include "NymRemEncChainDlgs.h"
#include "RBLegWins.h"
#include "RBBrkWins.h"
#include "RBCapWins.h"
//---------------------------------------------------------------------------
//#pragma link "explbtn"
#pragma resource "*.dfm"
TNymRemEncChainDlg *NymRemEncChainDlg;
//---------------------------------------------------------------------------

bool __fastcall EditRemixEncryptChain(TForm *Owner, const AnsiString &Title,
   AnsiString &Chain)
{
   assert(Chain != "");

   NymRemEncChainDlg = new TNymRemEncChainDlg(Owner);
   NymRemEncChainDlg->Caption = Title;
   NymRemEncChainDlg->chain = Chain;
   NymRemEncChainDlg->ShowModal();

   bool ok = NymRemEncChainDlg->ModalResult == mrOk;
   if (ok)
   {
      Chain = NymRemEncChainDlg->Chain->Items->CommaText;
      int i = 1; while (i < Chain.Length())
      {
         if (Chain[i] == ',') // add space after each comma
         {
            Chain.Insert(' ', i + 1);
            i++;
         }
         i++;
      }
   }

   delete NymRemEncChainDlg;
   return ok;
}

//---------------------------------------------------------------------------

__fastcall TNymRemEncChainDlg::TNymRemEncChainDlg(TComponent* Owner)
        : TForm(Owner)
{
   rlist = new TStringList();
   AnsiString rlisttxt = MixDir() + "rlist.txt";
   if (FileExists(rlisttxt))
      rlist->LoadFromFile(rlisttxt);

   capstrings = new TStringList();
   GetCapStrings(rlist, capstrings);

   brokenchains = new TStringList();
   GetBrokenCpunkChains(rlist, brokenchains);

   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, RemailerBox->Font, NULL);
   RemailerBox->Color = color;
   RemailerList->Font->Assign(RemailerBox->Font);
   RemailerList->Color = color;
   Chain->Font->Assign(RemailerBox->Font);
   Chain->Color = color;
}
//---------------------------------------------------------------------------

__fastcall TNymRemEncChainDlg::~TNymRemEncChainDlg()
{
   delete brokenchains;
   delete capstrings;
   delete rlist;
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::GetBrokenCpunkChains(TStrings *Rlist, TStrings *Broken)
{
   Broken->Clear();

   bool saving = false;
   for (int i = 0; i < Rlist->Count; i++)
   {
      AnsiString line = Rlist->Strings[i];
      if (line.Pos("Broken type-I remailer chains:") == 1)
      {
         saving = true;
         continue;
      }

      if (saving && line.Trim().IsEmpty())
         break;

      if (saving)
      {
         int pos;
         while ((pos = line.Pos("(")) > 0) line.Delete(pos, 1);
         while ((pos = line.Pos(")")) > 0) line.Delete(pos,1);
         while ((pos = line.Pos(" ")) > 0) line[pos] = ',';

         Broken->Add(line);
      }
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::GetCapStrings(TStrings *Rlist, TStrings *CapStrings)
{
   // get remailer capstrings first so GetCaps for nymservers below works
   for (int i = 0; i < Rlist->Count; i++)
   {
      AnsiString line = Rlist->Strings[i].Trim();
      if (line.Pos("$remailer{") == 1)
         CapStrings->Add(line);
   }
   // construct nymserver caps and combine
   AnsiString nymsrvfile = AppDir() + "nymserver.asc";
   if (FileExists(nymsrvfile))
   {
      TStringList *nymsrv = new TStringList();
      nymsrv->LoadFromFile(nymsrvfile);

      for (int i = 0; i < nymsrv->Count; i++)
      {
         AnsiString line = nymsrv->Strings[i].Trim();
         //if (line.Pos("-----BEGIN PGP PUBLIC KEY BLOCK-----") == 1)
         //   break;

         if (line.Pos("$nymserver{") == 1)
         {
            AnsiString remailer = GetTok(1, line, ';').Trim();
            CapStrings->Add(GetTok(0, line, ';').Trim() + " " + GetCaps(remailer));
         }
      }
      delete nymsrv;
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::RemailerListClear()
{
   RemailerList->Clear();
   RemailerList->Color = clBtnFace;
   CopyBtn->Enabled = false;
   //LegendBtn->Enabled = false;
   //Label6->Caption = "";
}
//---------------------------------------------------------------------------

/*void __fastcall TNymRemEncChainDlg::LoadRemailers(const AnsiString &delremname)
{
   AnsiString cap = Caption;
   Caption = "Loading...";
   Snooze(10);

   LoadRemailersAll();

   if (delremname != "random")
   for (int i = 0; i < RemailerList->Items->Count; i++)
   {
      AnsiString name = GetTok(0, RemailerList->Items->Strings[i], ' ');
      if (name == delremname)
      {
         RemailerList->Items->Delete(i);
         break;
      }
   }

   RemailerList->Items->Insert(0, "random");
   //LegendBtn->Enabled = true;
   CopyBtn->Enabled = false;
   Caption = cap;
}*/
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::LoadRemailersAll()
{
   AnsiString rembefore, remafter;
   GetRemailersBeforeAndAfter(rembefore, remafter);

   RemailerList->Clear();
   Snooze(50);

   RemailerList->Items->Add("random");
   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString line = rlist->Strings[i];
      if (line.Pos("%") > 0)
      {
         AnsiString remailer = GetTok(0, line, ' ');

         if (RemailerPrecedesOrFollowsSelf(remailer, rembefore, remafter))
            continue;
         if (remailer == rembefore)
            continue;

         int p;
         if (RemailerIsBroken(remailer, rembefore, remafter, p))
            FixBrokenLine(line, p);

         RemailerList->Items->Add(line);
      }
   }

   CopyBtn->Enabled = false;
}
//---------------------------------------------------------------------------

bool __fastcall TNymRemEncChainDlg::RemailerIsBroken(const AnsiString &remailer,
   const AnsiString &rembefore, const AnsiString &remafter, int &X)
{
   if (IsBroken(remailer, "*") && IsBroken("*", remailer))
   {
      // everything is broken from this and it is NOT the last remailer
      X = 2;
      return true;
   }
   if (IsBroken(remailer, "*") && !remafter.IsEmpty())
   {
      // everything is broken from this and it is NOT the last remailer
      X = 1;
      return true;
   }

   if (IsBroken("*", remailer) && !rembefore.IsEmpty())
   {
      // everything is broken to this and it is NOT first
      X = -1;
      return true;
   }

   if (IsBroken(rembefore, remailer))
   {
      // broken with the one before
      X = -1;
      return true;
   }

   if (IsBroken(remailer, remafter))
   {
      // broken with the one after
      X = 1;
      return true;
   }

   return false;
}
//---------------------------------------------------------------------------

bool __fastcall TNymRemEncChainDlg::IsBroken(const AnsiString &Remailer1, const AnsiString &Remailer2)
{
   return brokenchains->IndexOf(Remailer1 + "," + Remailer2) >= 0;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymRemEncChainDlg::FixBrokenLine(AnsiString &line, int p)
{
   AnsiString broken;
   if (p <  0) broken = "<broken     "; else
   if (p == 1) broken = " broken>    "; else
   broken = "<broken>    ";

   AnsiString fixed = line.SubString(1, 42) + broken + line.SubString(55, 32);
   line = fixed;
   return line;
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::GetRemailersBeforeAndAfter(AnsiString &RemBefore,
   AnsiString &RemAfter)
{
   int idx = Chain->ItemIndex;

   RemBefore = "";
   RemAfter = "";

   if (idx > 0)
      RemBefore = GetRemailerAt(idx - 1);

   RemAfter = GetRemailerAt(idx);
}
//---------------------------------------------------------------------------

bool __fastcall TNymRemEncChainDlg::RemailerPrecedesOrFollowsSelf(const AnsiString &Remailer,
   const AnsiString &RemBefore, const AnsiString &RemAfter)
{
   return Remailer == RemBefore || Remailer == RemAfter;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymRemEncChainDlg::GetRemailerAt(int Position)
{
   assert(Position >= 0);

   AnsiString remailer = Chain->Items->Strings[Position];

   return remailer;
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::FormShow(TObject *)
{
   Chain->Items->CommaText = chain;
   Chain->ItemIndex = Chain->Items->Count - 1;
   //LoadRemailers("");
   LoadRemailersAll();
}
//---------------------------------------------------------------------------
void __fastcall TNymRemEncChainDlg::CopyBtnClick(TObject *)
{
   assert(RemailerList->ItemIndex >= 0);

   AnsiString remailerline = RemailerList->Items->Strings[RemailerList->ItemIndex];
   AnsiString remailer = GetTok(0, remailerline, ' ');

   if (remailerline.Pos("broken") > 0)
   {
      MessageBeep(MB_ICONHAND);
      return;
   }
   if (remailer != "random" && RemailerInChain(remailer))
   {
      if (MsgBox(this, "This remailer is already in your chain. Use it again?   ",
         MT_CONFIRM, MB_OKCANCEL) == IDCANCEL)
      {
         return;
      }
   }

   Chain->Items->Insert(Chain->ItemIndex, remailer);
   //LoadRemailersAll(remailer);
   LoadRemailersAll();
}
//---------------------------------------------------------------------------

bool __fastcall TNymRemEncChainDlg::RemailerInChain(const AnsiString &Remailer)
{
   if (Chain->Items->Strings[0].Pos("(" + Remailer + ")") > 0)
      return true;

   return Chain->Items->IndexOf(Remailer) >= 0;
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::DelRemBtnClick(TObject *)
{
   int idx = Chain->ItemIndex;
   assert(idx >= 0);

   Chain->Items->Delete(Chain->ItemIndex);
   if (idx >= Chain->Items->Count - 1)
      idx--;
   if (idx >= 0)
   {
      Chain->ItemIndex = idx;
      DelRemBtn->Enabled = true;
   }
   else
      DelRemBtn->Enabled = false;
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::LegendBtnClick(TObject *)
{
   OpenLegendToolWindow(this, Left + LegendBtn->Left, Top + LegendBtn->Top + 25);
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::BrokeBtnClick(TObject *)
{
   OpenBrokenChainsWindow(this);
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::RemailerListClick(TObject *)
{
   CopyBtn->Enabled = RemailerList->ItemIndex >= 0;
   Hint = " " + GetCurrentCaps();

   AnsiString remailer = GetTok(0, RemailerList->Items->Strings[RemailerList->ItemIndex], ' ');
   if (RBCapWin != NULL)
   {
      OpenCapsToolWindow(this, remailer, Hint, Left + CapBtn->Left,
         Top + CapBtn->Top + 25);
   }
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymRemEncChainDlg::GetCurrentCaps()
{
   int idx = RemailerList->ItemIndex;
   if (idx < 0)
      return "";

   AnsiString rem = GetTok(0, RemailerList->Items->Strings[idx], ' ').Trim();
   return GetCaps(rem).Trim();
}
//---------------------------------------------------------------------------

bool __fastcall TNymRemEncChainDlg::HasCap(const AnsiString &Remailer, const AnsiString &Cap)
{
   bool has = false;
   AnsiString caps = GetCaps(Remailer);

   if (caps.Pos(" " + Cap + " ") > 0)
      has = true;

   return has;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymRemEncChainDlg::GetCaps(const AnsiString &Remailer)
{
   AnsiString caps, line, rem = Remailer;
   //if (Remailer.Pos("(") > 0)
   //   rem = GetTok(0, GetTok(1, Remailer, '('), ')');

   for (int i = 0; i < capstrings->Count; i++)
   {
      line = capstrings->Strings[i];
      if (line.Pos("$remailer{") != 1)
          continue;

      if (GetTok(1, line, '\"') == rem)
      {
         caps = GetTok(1, line, '>');
         caps = caps.SubString(1, caps.Length() - 2);
         break;
      }
   }

   return caps;
}
//---------------------------------------------------------------------------

int __fastcall TNymRemEncChainDlg::UpDayCount(const AnsiString &Remailer)
{
   AnsiString caps = GetCaps(Remailer);
   return GetTok(1, caps, '|').Length();
}
//---------------------------------------------------------------------------

/*AnsiString __fastcall TNymRemEncChainDlg::GetHistory(const AnsiString &Remailer)
{
   AnsiString history;

   for (int i = 0; i < rlist->Count; i++)
   {
      AnsiString line = rlist->Strings[i];
      if (line.Pos(Remailer + " ") == 1 && line.Pos("%") > 0)
      {
         history = line.SubString(43, 12).Trim();
         int p; while((p = line.Pos(" ")) >0)
            line.Delete(p, 1);
         break;
      }
   }

   return history;
}*/
//---------------------------------------------------------------------------

/*void __fastcall TRBWizardDlg::ListBox1DrawItem(TWinControl *, int Index,
   TRect &Rect, TOwnerDrawState)
{
   TCanvas *canvas = ListBox1->Canvas;
   AnsiString spec, line = ListBox1->Items->Strings[Index];
   AnsiString orgname = ListBox1->Font->Name, name = GetTok(0, line, ' ');
   int orgsize = ListBox1->Font->Size;

   if (!HasCap(name, "latent") || !HasCap(name, "ek") || line.Pos("broken") > 0)
      canvas->Font->Color = clGray;

   bool broke = line.Pos("broken") > 0;
   if (broke)
      canvas->Font->Style = TFontStyles() << fsStrikeOut;

   canvas->FillRect(Rect);
   canvas->TextOut(Rect.Left + 3, Rect.Top, line.SubString(1, 9).Trim());

   if (broke)
      canvas->Font->Style = TFontStyles();

   canvas->Font->Name = "Courier New";
   canvas->Font->Size = 8;

   canvas->TextOut(Rect.Left + 63, Rect.Top, line.SubString(43, 12));
   canvas->Font->Name = orgname;
   canvas->Font->Size = orgsize;
   spec = line.SubString(56, 9);
   canvas->TextOut(Rect.Left + 195 - canvas->TextWidth(spec), Rect.Top, spec);
   spec = line.SubString(65, 7);
   canvas->TextOut(Rect.Left + 245 - canvas->TextWidth(spec), Rect.Top, spec);
}*/
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::RemailerListDrawItem(TWinControl *,
        int Index, TRect &Rect, TOwnerDrawState)
{
   TCanvas *canvas = RemailerList->Canvas;
   AnsiString spec, line = RemailerList->Items->Strings[Index];
   AnsiString orgname = RemailerList->Font->Name, name = GetTok(0, line, ' ');
   int orgsize = RemailerList->Font->Size;

   if (name != "random" && (!HasCap(name, "latent") || !HasCap(name, "ek") || line.Pos("broken") > 0))
      canvas->Font->Color = clGrayText;

   bool broke = line.Pos("broken") > 0;
   if (broke)
      canvas->Font->Style = TFontStyles() << fsStrikeOut;

   canvas->FillRect(Rect);
   canvas->TextOut(Rect.Left + 3, Rect.Top, line.SubString(1, 9).Trim());

   if (broke)
      canvas->Font->Style = TFontStyles();

   canvas->Font->Name = "Courier New";
   canvas->Font->Size = 8;

   canvas->TextOut(Rect.Left + 53, Rect.Top, line.SubString(43, 12));
   canvas->Font->Name = orgname;
   canvas->Font->Size = orgsize;
   spec = line.SubString(56, 9);
   canvas->TextOut(Rect.Left + 185 - canvas->TextWidth(spec), Rect.Top, spec);
   spec = line.SubString(65, 16);
   canvas->TextOut(Rect.Left + 230 - canvas->TextWidth(spec), Rect.Top, spec);
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::ChainClear()
{
   for (int i = 0; i < Chain->Items->Count; i++)
      delete Chain->Items->Objects[i];
   Chain->Clear();
}
//---------------------------------------------------------------------------

int __fastcall TNymRemEncChainDlg::ChainLength()
{
   //assert(Length->Text.ToIntDef(0) >= 0 && Length->Text.ToIntDef(0) <= 20);
   //return Length->Text.ToInt();
   return 0;
}
//---------------------------------------------------------------------------

AnsiString __fastcall TNymRemEncChainDlg::LastInChain()
{
   assert(Chain->Items->Count >= 2);

   if (Chain->Items->Count <= 2)
      return "";

   return Chain->Items->Strings[Chain->Items->Count - 2];
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::ChainClick(TObject *)
{
   DelRemBtn->Enabled =
      Chain->ItemIndex >= 0 && Chain->ItemIndex < Chain->Items->Count - 1;
   LoadRemailersAll();
}
//---------------------------------------------------------------------------

void __fastcall TNymRemEncChainDlg::CapBtnClick(TObject *)
{
   AnsiString remailer, caps;
   if (RemailerList->ItemIndex >= 0)
   {
      remailer = GetTok(0, RemailerList->Items->Strings[RemailerList->ItemIndex], ' ');
      caps = GetCurrentCaps();
   }
   else
   {
      remailer = "nothing selected";
      caps = " Select a remailer to view it's capstring.";
   }

   OpenCapsToolWindow(this, remailer, caps, Left + CapBtn->Left,
      Top + CapBtn->Top + 25
   );
}
//---------------------------------------------------------------------------

