/*
*  NymRptsDlgs.cpp
*  Copyright  1999-2005 by Richard Christman
*  ALL RIGHTS RESERVED
*
*  This source file is released for peer review purposes only. It is not public domain. 
*  Redistribution either in part or whole is forbidden. The sole source for QuickSilver 
*  distribution currently is http://quicksilvermail.net . You have permission to compile
*  this source for your own use.
*
*  Richard
*  --
*  R.Christman
*  quicksilver@quicksilvermail.net
*  http://quicksilvermail.net
*
*/
//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

#include "NymRptsDlgs.h"
#include "ViewRptsDlgs.h"
#include "Global2.h"

const AnsiString arrow = " -> ";
//---------------------------------------------------------------------------
#pragma resource "*.dfm"
TNymRptsDlg *NymRptsDlg;
//---------------------------------------------------------------------------
void __fastcall RunNewNymReports(TForm *Owner)
{
   if (SecureModeMsg(Owner))
      return;

   NymRptsDlg = new TNymRptsDlg(Owner);
   NymRptsDlg->ShowModal();
   delete NymRptsDlg;
}
//---------------------------------------------------------------------------

__fastcall TNymRptsDlg::TNymRptsDlg(TComponent* Owner)
	: TForm(Owner)
{
   Global->DB->GetNyms(NymList->Items);

   TColor color;
   LoadFonts(AppIni(), NymIni, color, Font, NymList->Font, NULL);
   NymList->Color = color;
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::SelectAll(bool select)
{
   for (int i=0; i<NymList->Items->Count; i++)
      NymList->Selected[i] = select;
   OkBtn->Enabled = NymList->SelCount>0;
   ActiveControl = Panel1;
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::NymListDblClick(TObject *Sender)
{
   ModalResult = mrOk;
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::SelectAllBtnClick(TObject *Sender)
{
   SelectAll(true);
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::FormCloseQuery(TObject *Sender, bool &CanClose)
{
   if (ModalResult==mrOk)
   {
      if (!NymList->SelCount)
      {
         MsgBox(this, "No nyms selected for report!", MT_ERROR, MB_OK);
         CanClose = false;
         return;
      }

      AnsiString cap = Caption; Caption = "Working..."; Sleep(100);

      if (ProfileBtn->Checked) GenProfileReport(); else
      if (ProfileKeysBtn->Checked) GenProfileKeysReport(); else
      if (EncryptBtn->Checked) GenEncryptReport(); else
      if (ChainsBtn->Checked) GenChainsReport(); else
      if (ExportBtn->Checked) GenExportReport(); else
      if (EncryptSubBtn->Checked) GenEncryptSubReport();
   }
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::GenChainsReport()
{
   if (SecureModeMsg(this))
      return;

   AnsiString nym, line, chain;
   TStringList *list = new TStringList();
   TStringList *profile = new TStringList();

   for (int n=0; n<NymList->Items->Count; n++)
   {
      if (!NymList->Selected[n])
         continue;

      nym = NymList->Items->Strings[n];
      list->Add(GetTok(0, NymList->Items->Strings[n], '\255'));
      Global->DB->GetNym(nym, profile);

      bool inrepblk = false;
      for (int i = 0; i < profile->Count; i++)
      {
         line = profile->Strings[i].Trim();

         if (line.Pos("Public Key:") == 1)
            break;

         if (!inrepblk && line.Pos("Reply-Block:") != 1)
            continue;

         if (line.Pos("Reply-Block:") == 1)
         {
            inrepblk = true;
            chain = "";
            continue;
         }

         if (!line.IsEmpty() && line.Pos(":") == 0)
         {
            if (!chain.IsEmpty())
               chain += arrow;
            chain += line;
            continue;
         }

         if (line.IsEmpty())
         {
            inrepblk = false;
            list->Add(chain);
            chain = "";
         }
      }

      list->Add("");
   }

   ViewNymReport(this, " Nym Remailer Chains", list);

   delete profile;
   delete list;
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::GenEncryptSubReport()
{
   if (SecureModeMsg(this))
      return;

   AnsiString nym, line, esub, thesub, group;
   TStringList *list = new TStringList();
   TStringList *profile = new TStringList();

   for (int n=0; n<NymList->Items->Count; n++)
   {
      if (!NymList->Selected[n])
         continue;

      nym = NymList->Items->Strings[n];
      list->Add(GetTok(0, NymList->Items->Strings[n], '\255'));
      Global->DB->GetNym(nym, profile);

      bool inrepblk = false;
      for (int i=0; i<profile->Count; i++)
      {
         line = profile->Strings[i].Trim();

         if (line.Pos("Public Key:") == 1)
            break;

         if (!inrepblk && line.Pos("Reply-Block:") != 1)
            continue;

         if (line.Pos("Reply-Block:") == 1 || line.Pos(":") == 0)
         {
            if (!esub.IsEmpty() && !thesub.IsEmpty() && !group.IsEmpty())
            {
               list->Add(group + "; esub=" + esub + "; " + thesub);
               group = ""; esub = ""; thesub = "";
               continue;
            }
         }

         if (line.Pos("Encrypt-Subject: ")==1) esub = line.SubString(18,256); else
         if (line.Pos("Newsgroups: ")==1) group = line.SubString(13,256); else
         if (line.Pos("Subject: ")==1) thesub = line.SubString(10,256);
         inrepblk = true;
      }

      list->Add("");
   }

   ViewNymReport(this, " Nym Encrypt Subjects", list);

   delete profile;
   delete list;
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::GenEncryptReport()
{
   if (SecureModeMsg(this))
      return;

   AnsiString nym, line, keyline;
   TStringList *list = new TStringList();
   TStringList *profile = new TStringList();

   for (int n=0; n<NymList->Items->Count; n++)
   {
      if (!NymList->Selected[n])
         continue;

      nym = NymList->Items->Strings[n];
      list->Add(GetTok(0, NymList->Items->Strings[n],'\255'));
      Global->DB->GetNym(nym, profile);

      bool inrepblk = false;
      for (int i=0; i<profile->Count; i++)
      {
         line = profile->Strings[i].Trim();

         if (line.Pos("Public Key:") == 1)
            break;

         if (!inrepblk && line.Pos("Reply-Block:") != 1)
            continue;

         if (line.Pos("Reply-Block:") == 1)
         {
            inrepblk = true;
            keyline = "";
            continue;
         }

         if (line.Pos("Encrypt-Key:") == 1)
         {
            if (!keyline.IsEmpty())
               keyline += arrow;
            keyline += line.SubString(14, 128);
            continue;
         }

         if (line.IsEmpty())
         {
            inrepblk = false;
            list->Add(keyline);
            keyline = "";
         }
      }

      list->Add("");
   }

   ViewNymReport(this, " Nym Encrypt-Keys", list);

   delete profile;
   delete list;
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::GenExportReport()
{
   if (SecureModeMsg(this))
      return;

   AnsiString nym, line, lastline;
   TStringList *list = new TStringList();
   TStringList *profile = new TStringList();

   for (int n=0; n<NymList->Items->Count; n++)
   {
      if (!NymList->Selected[n])
         continue;

      nym = NymList->Items->Strings[n];

      if (list->Count > 0)
         list->Add("");

      list->Add(GetTok(0, NymList->Items->Strings[n],'\255'));
      Global->DB->GetNym(nym, profile);
      list->Add("");

      bool inkeys = false;
      for (int i=0; i<profile->Count; i++)
      {
         line = profile->Strings[i].Trim();

         if (line.Pos("Public Key:") == 1)
         {
            inkeys = true;
         }

         if (inkeys)
            list->Add(line);

         if (inkeys && line.Pos("-----END PGP PRIVATE KEY BLOCK-----") == 1)
            break;
      }
   }

   ViewNymReport(this, " Nym PGP Keys", list);

   delete profile;
   delete list;
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::GenProfileReport()
{
   if (SecureModeMsg(this))
      return;

   AnsiString nym, item, line;
   TStringList *list = new TStringList();
   TStringList *profile = new TStringList();

   for (int n=0; n<NymList->Items->Count; n++)
   {
      if (!NymList->Selected[n])
         continue;

      nym = NymList->Items->Strings[n];
      Global->DB->GetNym(nym, profile);

      for (int i=0; i<profile->Count; i++)
      {
         line = profile->Strings[i];
         if (line.Pos("Public Key:") == 1)
            break;

         list->Add(line);
      }
   }

   ViewNymReport(this, " Nym Profiles", list);

   delete profile;
   delete list;
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::GenProfileKeysReport()
{
   if (SecureModeMsg(this))
      return;

   AnsiString nym, item, line;
   TStringList *list = new TStringList();
   TStringList *profile = new TStringList();

   for (int n=0; n<NymList->Items->Count; n++)
   {
      if (!NymList->Selected[n])
         continue;

      nym = NymList->Items->Strings[n];
      Global->DB->GetNym(nym, profile);

      for (int i = 0; i < profile->Count; i++)
      {
         line = profile->Strings[i];

         list->Add(line);

         if (line.Pos("-----END PGP PRIVATE KEY BLOCK-----") == 1)
            break;
      }

      list->Add("");
   }

   ViewNymReport(this, " Nym Profiles w/Keys", list);

   delete profile;
   delete list;
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::NymListClick(TObject *Sender)
{
   OkBtn->Enabled = NymList->SelCount>0;
}
//---------------------------------------------------------------------------

void __fastcall TNymRptsDlg::NymListDrawItem(TWinControl *Control, int Index,
        TRect &Rect, TOwnerDrawState State)
{
   TCanvas *Canvas = NymList->Canvas;
   Canvas->FillRect(Rect);
   AnsiString item = GetTok(0, NymList->Items->Strings[Index], '\255');
   Canvas->TextRect(Rect, Rect.Left + 1, Rect.Top + 1, item);
}
//---------------------------------------------------------------------------
